Module callback_worker

Module callback_worker 

Structs

IOWorkers
Example callback worker implement with crossfire::mpmc, can be shared among multiple driver instances.
Inline
Example Inline worker that executes callbacks directly without spawning threads. Use this for very lightweight callback logic to avoid thread context switching overhead.

Traits

Worker
A trait for workers that accept IO events.