Inline

Struct Inline 

pub struct Inline;
Example Inline worker that executes callbacks directly without spawning threads. Use this for very lightweight callback logic to avoid thread context switching overhead.

Safety

It does not check and resubmit short I/O