pub struct InlineClosure<C: CbArgs>(pub Box<dyn Fn(C, i64, Result<Option<Buffer>, Errno>) + Send>);Expand description
Example Inline worker that executes callbacks directly without spawning threads. Use this for very lightweight callback logic to avoid thread context switching overhead.
Safety
It does not resubmit short I/O
Tuple Fields
0: Box<dyn Fn(C, i64, Result<Option<Buffer>, Errno>) + Send>Trait Implementations